<?php
/*
Plugin Name: ZimmWriter UnEscape
Description: Automatically unescapes HTML entities in post titles, content, excerpts, categories, and tags when created via REST API
Version: 1.7
Author: Your Name
*/

// Prevent direct access to this file
if (!defined('ABSPATH')) {
    exit;
}

class HTML_Entity_Unescape {
    // Array of taxonomies to process
    private $taxonomies = array('category', 'post_tag');

    public function __construct() {
        // Hook into REST API post creation
        add_action('rest_after_insert_post', array($this, 'unescape_post_content'), 10, 3);
        
        // Hook into REST API term creation (for both categories and tags)
        add_action('rest_after_insert_term', array($this, 'unescape_term_name'), 10, 3);
    }

    /**
     * Unescapes HTML entities in post title, content, and excerpt
     * 
     * @param WP_Post $post The post object
     * @param WP_REST_Request $request The request object
     * @param bool $creating Whether this is a new post being created
     */
    public function unescape_post_content($post, $request, $creating) {
        // Only process new posts
        if (!$creating) {
            return;
        }

        // Get the post ID
        $post_id = $post->ID;

        // Get the current post title, content, and excerpt
        $title = get_the_title($post_id);
        $content = get_post_field('post_content', $post_id);
        $excerpt = get_the_excerpt($post_id);

        // Unescape HTML entities
        $unescaped_title = html_entity_decode($title, ENT_QUOTES | ENT_HTML5, 'UTF-8');
        $unescaped_content = html_entity_decode($content, ENT_QUOTES | ENT_HTML5, 'UTF-8');
        $unescaped_excerpt = html_entity_decode($excerpt, ENT_QUOTES | ENT_HTML5, 'UTF-8');

        // Update the post if changes were made
        if ($title !== $unescaped_title || 
            $content !== $unescaped_content || 
            $excerpt !== $unescaped_excerpt) {
            
            // Remove the action temporarily to prevent infinite loop
            remove_action('rest_after_insert_post', array($this, 'unescape_post_content'));

            $post_data = array(
                'ID' => $post_id,
                'post_title' => $unescaped_title,
                'post_content' => $unescaped_content
            );

            if (!empty($excerpt)) {
                $post_data['post_excerpt'] = $unescaped_excerpt;
            }

            wp_update_post($post_data);

            // Re-add the action
            add_action('rest_after_insert_post', array($this, 'unescape_post_content'), 10, 3);
        }
    }

    /**
     * Unescapes HTML entities in term names (categories and tags)
     * 
     * @param WP_Term $term The term object
     * @param WP_REST_Request $request The request object
     * @param bool $creating Whether this is a new term being created
     */
    public function unescape_term_name($term, $request, $creating) {
        // Only process new terms
        if (!$creating) {
            return;
        }

        // Only process specified taxonomies
        if (!in_array($term->taxonomy, $this->taxonomies)) {
            return;
        }

        // Get the current term name
        $name = $term->name;

        // Unescape HTML entities
        $unescaped_name = html_entity_decode($name, ENT_QUOTES | ENT_HTML5, 'UTF-8');

        // Update the term if changes were made
        if ($name !== $unescaped_name) {
            // Remove the action temporarily to prevent infinite loop
            remove_action('rest_after_insert_term', array($this, 'unescape_term_name'));

            wp_update_term($term->term_id, $term->taxonomy, array(
                'name' => $unescaped_name
            ));

            // Re-add the action
            add_action('rest_after_insert_term', array($this, 'unescape_term_name'), 10, 3);
        }
    }
}

// Initialize the plugin
new HTML_Entity_Unescape();